import os

import kubernetes
import requests


async def initialize_game_token(communicator):
    """Gets game token and stores it somewhere accesible."""
    if os.environ["WORKER"] == "kubernetes":
        api = kubernetes.client.CoreV1Api()
        game_id = os.environ.get("GAME_ID")

        secret = v1.read_namespaced_secret(f"game-{game_id}-token", "default")
        os.environ["TOKEN"] = secret.data["token"]

    await communicator.patch_token({"token": os.environ["TOKEN"]})



import mock
from unittest import TestCase
from requests import Response
from simulation.workers.worker import Worker
from tests.test_simulation.concrete_worker import ConcreteWorker


DEFAULT_RESPONSE_CONTENT = (
    b'{"action": "test_action",' b'"log": "test_log",' b'"avatar_updated": "True"}'
)

MISSING_KEY_RESPONSE_CONTENT = (
    b'{"corruptedKey": "test_action",'
    b'"log": "test_log",'
    b'"avatar_updated": "True"}'
)


def construct_test_response(status_code=200, response_content=DEFAULT_RESPONSE_CONTENT):
    post_response = Response()
    post_response.status_code = status_code
    post_response._content = response_content
    return post_response


class TestWorker(TestCase):
    def setUp(self):
        self.worker = ConcreteWorker(1, 0)

    @mock.patch(
        "simulation.workers.worker.requests.post",
        return_value=construct_test_response(),
    )
    def test_fetch_data_fetches_correct_response(self, mocked_post):
        self.worker.fetch_data(state_view={})

        mocked_post.assert_called_once()
        self.assertEqual(self.worker.serialized_action, "test_action")
        self.assertEqual(self.worker.log, "test_log")
        self.assertEqual(self.worker.has_code_updated, "True")

    def test_setting_defaults_works_correctly(self):
        self.worker.log = "test_log_fake"
        self.worker.serialised_action = "test_action_fake"
        self.worker.has_code_updated = "test_avatar_updated_fake"

        self.worker._set_defaults()

        self.assertIsNone(self.worker.serialized_action)
        self.assertIsNone(self.worker.log)
        self.assertFalse(self.worker.has_code_updated)

    @mock.patch(
        "simulation.workers.worker.requests.post",
        return_value=construct_test_response(status_code=500),
    )
    @mock.patch.object(target=Worker, attribute="_set_defaults")
    def test_fetch_data_cannot_connect_to_worker(
        self, mocked_set_defaults, mocked_post
    ):
        self.worker.fetch_data(state_view={})

        mocked_post.assert_called_once()
        mocked_set_defaults.assert_called_once()

    @mock.patch(
        "simulation.workers.worker.requests.post",
        return_value=construct_test_response(
            response_content=MISSING_KEY_RESPONSE_CONTENT
        ),
    )
    @mock.patch.object(target=Worker, attribute="_set_defaults")
    def test_missing_key_in_worker_data(self, mocked_set_defaults, mocked_post):
        self.worker.fetch_data(state_view={})

        mocked_post.assert_called_once()
        mocked_set_defaults.assert_called_once()



class Avatar:
    def next_turn(self, world_state, avatar_state):
        self.world_state = world_state
        self.avatar_state = avatar_state

        if world_state.get_cell(avatar_state.location).generates_score:
            return WaitAction()

        possible_directions = self.get_possible_directions()
        directions_to_emphasise = [
            d
            for d in possible_directions
            if self.is_towards(d, self.get_closest_pickup_location())
        ]
        return MoveAction(
            random.choice(possible_directions + (directions_to_emphasise * 5))
        )

    def is_towards(self, direction, location):
        if location:
            return self.distance_between(
                self.avatar_state.location, location
            ) > self.distance_between(self.avatar_state.location + direction, location)
        else:
            return None

    def distance_between(self, a, b):
        return abs(a.x - b.x) + abs(a.y - b.y)

    def get_closest_pickup_location(self):
        pickup_cells = list(self.world_state.pickup_cells())
        if pickup_cells:
            c = min(
                pickup_cells,
                key=lambda cell: self.distance_between(
                    cell.location, self.avatar_state.location
                ),
            )
            print("targetting" + c)
            return c.location
        else:
            return None

    def get_possible_directions(self):
        directions = (direction.EAST, direction.SOUTH, direction.WEST, direction.NORTH)
        return [
            d
            for d in directions
            if self.world_state.can_move_to(self.avatar_state.location + d)
        ]



from functools import wraps

from rest_framework import authentication, permissions


def default_preview_user(view_func):
    @wraps(view_func)
    def wrapped(request, *args, **kwargs):
        return view_func(request, *args, **kwargs)

    return wrapped


class CsrfExemptSessionAuthentication(authentication.SessionAuthentication):
    def enforce_csrf(self, request):
        return False


class GameHasToken(permissions.BasePermission):
    """
    Used to verify that an incoming request has permission
    to access a given object from the models.

    This is done on a per object basis. The object must have an `auth_token`
    attribute to be used with this permission class.
    """

    def has_permission(self, request, view):
        return True

    def has_object_permission(self, request, view, obj):
        return self.check_for_token(request, obj)

    def check_for_token(self, request, obj):
        try:
            return (
                obj.auth_token == ""
                or request.META["HTTP_GAME_TOKEN"] == obj.auth_token
            )
        except (KeyError, AttributeError):
            return False
